@@ -40,7 +40,7 @@ if (.Platform$OS.type == "windows") {
             dsp <- Sys.getenv("DISPLAY")
             if(.Platform$OS.type == "windows") windows
             else if (.Platform$GUI == "AQUA" ||
-                     ((!nzchar(dsp) || grepl("^/tmp/launch-", dsp))
+                     ((!nzchar(dsp) || grepl("^/tmp/launch-|^/private/tmp/com.apple.launchd", dsp))
                       && .Call(C_makeQuartzDefault))) quartz
             else if (nzchar(dsp) && .Platform$GUI %in% c("X11", "Tk")) X11
 	    else defdev
