@@ -42,9 +42,8 @@ cbind <- function(..., deparse.level = 1)
     while(na > 0 && is.null(argl[[na]])) { argl <- argl[-na]; na <- na - 1 }
     if(na == 0) return(NULL)
     if(na == 1) {
-	if(isS4(..1))
-	    return(cbind2(..1))
-	else return(.Internal(cbind(deparse.level, ...)))
+	if(isS4(..1)) return(cbind2(..1))
+	else return(.__H__.cbind(..., deparse.level = deparse.level))
     }
 
     ## else :  na >= 2
