@@ -31,6 +31,11 @@
 #include "nmath.h"
 #include "dpq.h"
 
+#ifdef DEBUG_qbeta
+/* for REprintf */
+# include <R_ext/Print.h>
+#endif
+
 /* set the exponent of accu to -2r-2 for r digits of accuracy */
 /*---- NEW ---- -- still fails for p = 1e11, q=.5*/
 
@@ -50,7 +55,8 @@
 double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 {
     int swap_tail, i_pb, i_inn;
-    double a, adj, logbeta, g, h, pp, p_, prev, qq, r, s, t, tx, w, y, yprev;
+    Rboolean warned = FALSE;
+    double a, la, adj, logbeta, g, h, pp, p_, prev, qq, r, s, t, tx, w, y, wprev;
     double acu;
     volatile double xinbta;
 
@@ -100,31 +106,53 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
     /* change tail if necessary;  afterwards   0 < a <= 1/2	 */
     if (p_ <= 0.5) {
 	a = p_;	pp = p; qq = q; swap_tail = 0;
+	la = lower_tail ? R_D_log(alpha) : R_D_LExp(alpha);
     } else { /* change tail, swap  p <-> q :*/
-	a = (!lower_tail && !log_p)? alpha : 1 - p_;
+	a = R_DT_CIv(alpha); // = 1 - p_ < 1/2
+	/* la := log(a), but without numerical cancellation: */
+	la = lower_tail ? R_D_LExp(alpha) : R_D_log(alpha);
 	pp = q; qq = p; swap_tail = 1;
     }
 
+#ifdef DEBUG_qbeta
+    REprintf("qbeta(%g, %g, %g, lower_t=%d, log_p=%d):%s\n"
+	     "  swap_tail=%d, la=%g: ",
+	     alpha, p,q, lower_tail, log_p,
+	     (log_p && (p_ == 0. || p_ == 1.)) ? (p_==0.?" p_=0":" p_=1") : "",
+	     swap_tail, la);
+#endif
     /* calculate the initial approximation */
 
     /* y := {fast approximation of} qnorm(1 - a) :*/
-    r = sqrt(-2 * log(a));
+    r = sqrt(-2 * la);
     y = r - (const1 + const2 * r) / (1. + (const3 + const4 * r) * r);
-    if (pp > 1 && qq > 1) {
+    if (pp > 1 && qq > 1) { // use  Carter(1947), see AS 109, remark '5.'
 	r = (y * y - 3.) / 6.;
 	s = 1. / (pp + pp - 1.);
 	t = 1. / (qq + qq - 1.);
 	h = 2. / (s + t);
 	w = y * sqrt(h + r) / h - (t - s) * (r + 5. / 6. - 2. / (3. * h));
+#ifdef DEBUG_qbeta
+	REprintf("p,q > 1 => w=%g", w);
+#endif
 	xinbta = pp / (pp + qq * exp(w + w));
     } else {
 	r = qq + qq;
 	t = 1. / (9. * qq);
 	t = r * R_pow_di(1. - t + y * sqrt(t), 3);
-	if (t <= 0.)
+#ifdef DEBUG_qbeta
+	REprintf("min(p,q) <= 1: t=%g", t);
+#endif
+	if (t <= 0.) {
+#ifdef DEBUG_qbeta
+	    REprintf(" t <= 0: log1p(-a)=%.15g\n", log1p(-a));
+#endif
 	    xinbta = 1. - exp((log1p(-a)+ log(qq) + logbeta) / qq);
-	else {
+	} else {
 	    t = (4. * pp + r - 2.) / t;
+#ifdef DEBUG_qbeta
+	    REprintf(" t >= 0:  new t = %g ( > 1 ?)\n", t);
+#endif
 	    if (t <= 1.)
 		xinbta = exp((log(a * pp) + logbeta) / pp);
 	    else
@@ -132,20 +160,7 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 	}
     }
 
-    /* solve for x by a modified newton-raphson method, */
-    /* using the function pbeta_raw */
-
-    r = 1 - pp;
-    t = 1 - qq;
-    yprev = 0.;
-    adj = 1;
-    /* Sometimes the approximation is negative! */
-    if (xinbta < lower)
-	xinbta = 0.5;
-    else if (xinbta > upper)
-	xinbta = 0.5;
-
-    /* Desired accuracy should depend on  (a,p)
+    /* Desired accuracy for Newton iterations (below) should depend on  (a,p)
      * This is from Remark .. on AS 109, adapted.
      * However, it's not clear if this is "optimal" for IEEE double prec.
 
@@ -153,11 +168,27 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 
      * NEW: 'acu' accuracy NOT for squared adjustment, but simple;
      * ---- i.e.,  "new acu" = sqrt(old acu)
-
     */
     acu = fmax2(acu_min, pow(10.0, -13.0 - 2.5/(pp * pp) - 0.5/(a * a)));
-    tx = prev = 0.;	/* keep -Wall happy */
 
+#ifdef DEBUG_qbeta
+    REprintf(" -> xinbta = %.16g (Newton acu=%g)\n", xinbta, acu);
+#endif
+
+    /* solve for x by a modified newton-raphson method, */
+    /* using the function pbeta_raw */
+
+    r = 1 - pp;
+    t = 1 - qq;
+    wprev = 0.;
+    adj = 1;
+    /* Sometimes the approximation is negative! */
+    if (xinbta < lower)
+	xinbta = 0.5;
+    else if (xinbta > upper)
+	xinbta = 0.5;
+
+    tx = prev = 0.;	/* keep -Wall happy */
     for (i_pb=0; i_pb < 1000; i_pb++) {
 	y = pbeta_raw(xinbta, pp, qq, /*lower_tail = */ TRUE, FALSE);
 #ifdef IEEE_754
@@ -167,31 +198,48 @@ double qbeta(double alpha, double p, double q, int lower_tail, int log_p)
 #endif
 		ML_ERR_return_NAN;
 
-	y = (y - a) *
+	w = (y - a) *
 	    exp(logbeta + r * log(xinbta) + t * log1p(-xinbta));
-	if (y * yprev <= 0.)
+	if (w * wprev <= 0.)
 	    prev = fmax2(fabs(adj),fpu);
 	g = 1;
+#ifdef DEBUG_qbeta
+	REprintf("N(i=%d): x0=%.15g, pb(x0)=%.15g, w=%.15g, prev=%g,", i_pb, xinbta, y, w, prev);
+#endif
 	for (i_inn=0; i_inn < 1000;i_inn++) {
-	    adj = g * y;
+	    adj = g * w;
 	    if (fabs(adj) < prev) {
-		tx = xinbta - adj; /* trial new x */
-		if (tx >= 0. && tx <= 1) {
-		    if (prev <= acu)	goto L_converged;
-		    if (fabs(y) <= acu) goto L_converged;
+		tx = xinbta - adj; // x_{n+1} = x_n - g*w
+		if (0. <= tx && tx <= 1.) {
+		    if (prev <= acu || fabs(w) <= acu) {
+#ifdef DEBUG_qbeta
+			REprintf(" it{in}=%d, delta(x)=%g, %s <= acu  ==> convergence\n",
+				 i_inn, -adj, (prev <= acu) ? "prev" : "|w|");
+#endif
+			goto L_converged;
+		    }
 		    if (tx != 0. && tx != 1)
 			break;
 		}
 	    }
 	    g /= 3;
 	}
-	if (fabs(tx - xinbta) < 1e-15*xinbta) goto L_converged;
+#ifdef DEBUG_qbeta
+	REprintf(" it{in}=%d, delta(x)=%g\n", i_inn, -adj);
+#endif
+	if (fabs(tx - xinbta) <= 1e-15 * xinbta) // "<=" for xinbta == 0
+	    goto L_converged;
 	xinbta = tx;
-	yprev = y;
+	wprev = w;
     }
     /*-- NOT converged: Iteration count --*/
+    warned = TRUE;
     ML_ERROR(ME_PRECISION, "qbeta");
 
 L_converged:
+#ifdef DEBUG_qbeta
+    REprintf(" %s: Final delta(y) = %g\n",
+	     warned ? "_NO_ convergence" : "converged", y - a);
+#endif
     return swap_tail ? 1 - xinbta : xinbta;
 }
